@@ -1,6 +1,7 @@
 /*
  *  SPDX-FileCopyrightText: 2019 David Redondo <kde@david-redondo.de>
  *  SPDX-FileCopyrightText: 2015 Boudhayan Gupta <bgupta@kde.org>
+ *  SPDX-FileCopyrightText: 2025 Jhair Paris <dev@jhairparis.com>
  *
  *  SPDX-License-Identifier: LGPL-2.0-or-later
  */
@@ -10,24 +11,19 @@
 #include "CommandLineOptions.h"
 #include "ExportManager.h"
 #include "Geometry.h"
-#include "Gui/Annotations/AnnotationViewport.h"
-#include "Gui/Annotations/QmlPainterPath.h"
 #include "Gui/CaptureWindow.h"
 #include "Gui/Selection.h"
 #include "Gui/SelectionEditor.h"
 #include "Gui/SpectacleWindow.h"
-#include "Gui/ExportMenu.h"
-#include "Gui/HelpMenu.h"
-#include "Gui/OptionsMenu.h"
 #include "Gui/InlineMessageModel.h"
+#include "OcrManager.h"
 #include "Platforms/ImagePlatformXcb.h"
-#include "Platforms/VideoPlatform.h"
+#include "Platforms/PlatformLoader.h"
+#include "RecordingModeModel.h"
 #include "ShortcutActions.h"
 #include "PlasmaVersion.h"
 // generated
-#include "Config.h"
 #include "settings.h"
-#include "DebugUtils.h"
 
 #include <KFormat>
 #include <KGlobalAccel>
@@ -49,8 +45,10 @@
 #include <QDir>
 #include <QDrag>
 #include <QKeySequence>
+#include <QMetaObject>
 #include <QMimeData>
 #include <QMovie>
+#include <QObject>
 #include <QProcess>
 #include <QQmlComponent>
 #include <QQmlContext>
@@ -58,9 +56,10 @@
 #include <QScopedPointer>
 #include <QScreen>
 #include <QSystemTrayIcon>
+#include <QTemporaryFile>
+#include <QTextStream>
 #include <QTimer>
 #include <QtMath>
-#include <qobject.h>
 #include <qobjectdefs.h>
 
 using namespace Qt::StringLiterals;
@@ -68,6 +67,8 @@ using namespace Qt::StringLiterals;
 SpectacleCore *SpectacleCore::s_self = nullptr;
 static std::unique_ptr<KStatusNotifierItem> s_systemTrayIcon;
 
+static QList<KNotification *> notifications;
+
 SpectacleCore::SpectacleCore(QObject *parent)
     : QObject(parent)
 {
@@ -128,7 +129,6 @@ SpectacleCore::SpectacleCore(QObject *parent)
     // essential connections
     connect(SelectionEditor::instance(), &SelectionEditor::accepted,
             this, [this](const QRectF &rect, const ExportManager::Actions &actions){
-        ExportManager::instance()->updateTimestamp();
         m_returnToViewer = m_startMode == StartMode::Gui;
         if (m_videoMode) {
             const auto captureWindows = CaptureWindow::instances();
@@ -155,6 +155,7 @@ SpectacleCore::SpectacleCore(QObject *parent)
             // showViewerIfGuiMode(true);
             bool includePointer = m_cliOptions[CommandLineOptions::Pointer];
             includePointer |= m_startMode != StartMode::Background && Settings::videoIncludePointer();
+            ExportManager::instance()->updateTimestamp();
             const auto &output = m_outputUrl.isLocalFile() ? videoOutputUrl() : QUrl();
             static const auto rectKey = u"rect"_s;
             m_videoPlatform->startRecording(output, VideoPlatform::Region, {{rectKey, rect}}, includePointer);
@@ -163,7 +164,15 @@ SpectacleCore::SpectacleCore(QObject *parent)
             deleteWindows();
             m_annotationDocument->cropCanvas(rect);
             syncExportImage();
-            const auto &exportActions = actions & ExportManager::AnyAction ? actions : autoExportActions();
+            auto exportActions = actions & ExportManager::AnyAction ? actions : autoExportActions();
+            if (m_ocrExportInProgress) {
+                if (Settings::closeAfterOcr()) {
+                    exportActions.setFlag(ExportManager::Action::Save, false);
+                    exportActions.setFlag(ExportManager::Action::CopyPath, false);
+                    m_quitAfterOcr = true;
+                }
+                m_ocrExportInProgress = false;
+            }
             const bool willQuit = exportActions.testFlag(ExportManager::AnyAction) //
                 && exportActions.testFlag(ExportManager::UserAction) //
                 && Settings::quitAfterSaveCopyExport();
@@ -193,6 +202,8 @@ SpectacleCore::SpectacleCore(QObject *parent)
         setVideoMode(false);
         m_annotationDocument->clearAnnotations();
         m_annotationDocument->setBaseImage(image);
+        setExportImage(image);
+        ExportManager::instance()->updateTimestamp();
         SelectionEditor::instance()->reset();
         initCaptureWindows(CaptureWindow::Image);
         SpectacleWindow::setTitleForAll(SpectacleWindow::Unsaved);
@@ -527,7 +538,7 @@ SpectacleCore::SpectacleCore(QObject *parent)
                 // Not using xi18nc because it doesn't work with the link replacement logic.
                 // Also see https://invent.kde.org/graphics/spectacle/-/merge_requests/432#note_1111125
                 auto text = i18nc("@info", //
-                                  "QR Code found: %1",
+                                  "Code found: %1",
                                   result.toString().toHtmlEscaped().replace(urlRegex, linkifier));
                 return u"<html>" % text % u"</html>";
             }
@@ -537,6 +548,112 @@ SpectacleCore::SpectacleCore(QObject *parent)
     };
     connect(exportManager, &ExportManager::qrCodeScanned, this, onQRCodeScanned);
 
+    auto onOcrTextRecognized = [this](const QString &text, const QStringList &languageCodes, bool success) {
+        if (!success) {
+            InlineMessageModel::instance()->push(InlineMessageModel::Error, 
+                i18nc("@info", "Text extraction failed"));
+            return;
+        }
+        
+        if (text.isEmpty()) {
+            InlineMessageModel::instance()->push(InlineMessageModel::Copied, 
+                i18nc("@info", "No text found in the image"));
+            return;
+        }
+        
+        InlineMessageModel::instance()->push(InlineMessageModel::Copied, 
+            i18nc("@info", "Text extraction completed"));
+
+        // ensure program stays alive until the notification finishes.
+        if (!m_eventLoopLocker) {
+            m_eventLoopLocker = std::make_unique<QEventLoopLocker>();
+        }
+
+        auto notification = new KNotification(u"ocrTextExtracted"_s, KNotification::CloseOnTimeout, nullptr);
+        notification->setTitle(i18nc("@info:notification title", "Text Extracted"));
+
+        notifications.append(notification);
+
+        auto ocrManager = OcrManager::instance();
+        auto languageNames = ocrManager->availableLanguagesWithNames();
+
+        QStringList displayLanguages;
+        for (const QString &code : languageCodes) {
+            QString displayName = languageNames.value(code, code);
+            if (!displayLanguages.contains(displayName)) {
+                displayLanguages.append(displayName);
+            }
+        }
+
+        QString languagesText;
+        if (displayLanguages.size() == 1) {
+            languagesText = displayLanguages.first();
+        } else if (displayLanguages.size() == 2) {
+            languagesText = i18nc("@info The variables are language names, e.g. 'English and German'", "%1 and %2", displayLanguages.at(0), displayLanguages.at(1));
+        } else {
+            languagesText = displayLanguages.join(u", "_s);
+        }
+
+        auto notificationText = xi18nc("@info:notification", "Text copied to clipboard.<nl/>Languages used: %1", languagesText);
+        notification->setText(notificationText);
+        notification->setIconName(u"document-scan"_s);
+        
+        if (!text.isEmpty()) {
+            auto openEditorAction = notification->addAction(i18nc("@action:button", "Open in Text Editor"));
+            connect(openEditorAction, &KNotificationAction::activated, this, [text, notification]() {
+                // Create temporary file with extracted text 
+                auto exportManager = ExportManager::instance();
+                exportManager->updateTimestamp();
+                auto timestamp = exportManager->timestamp();
+                
+                QString filename = QStringLiteral("spectacle_ocr_%1.txt").arg(timestamp.toString(QStringLiteral("yyyyMMdd_HHmmss")));
+                QString templatePath = QDir::tempPath() + QStringLiteral("/") + filename;
+                
+                QTemporaryFile tempFile;
+                tempFile.setFileTemplate(templatePath);
+                tempFile.setAutoRemove(false);
+                
+                if (tempFile.open()) {
+                    QTextStream stream(&tempFile);
+                    stream << text;
+                    tempFile.close();
+                    
+                    auto job = new KIO::OpenUrlJob(QUrl::fromLocalFile(tempFile.fileName()));
+                    job->start();
+                }
+
+                notification->close();
+            });
+        }
+
+        auto onExpired = [this, notification] {
+            notifications.removeOne(static_cast<KNotification *>(notification));
+
+            if (notifications.empty() && m_eventLoopLocker) {
+                QTimer::singleShot(250, this, [this] {
+                    m_eventLoopLocker.reset();
+                });
+            }
+        };
+        connect(notification, &QObject::destroyed, this, onExpired);
+        // We keep the application running for a while to ensure the notification action remains available in the history.
+        // See: https://bugs.kde.org/show_bug.cgi?id=514434
+        QTimer::singleShot(180000, notification, onExpired);
+
+        notification->sendEvent();
+
+        if (m_quitAfterOcr) {
+            m_quitAfterOcr = false;
+            deleteWindows();
+        }
+    };
+
+    // Connect to OCR manager
+    connect(OcrManager::instance(), &OcrManager::textRecognized, this, onOcrTextRecognized);
+    connect(OcrManager::instance(), &OcrManager::statusChanged, this, [this](OcrManager::OcrStatus) {
+        Q_EMIT ocrStatusChanged();
+    });
+
     connect(exportManager, &ExportManager::errorMessage, this, &SpectacleCore::showErrorMessage);
 
     connect(m_annotationDocument.get(), &AnnotationDocument::repaintNeeded, m_annotationSyncTimer.get(), qOverload<>(&QTimer::start));
@@ -580,6 +697,89 @@ SpectacleCore::SpectacleCore(QObject *parent)
     });
 }
 
+bool SpectacleCore::ocrAvailable() const
+{
+    return OcrManager::instance()->isAvailable();
+}
+
+OcrManager::OcrStatus SpectacleCore::ocrStatus() const
+{
+    return OcrManager::instance()->status();
+}
+
+QVariantMap SpectacleCore::ocrAvailableLanguages() const
+{
+    auto ocrManager = OcrManager::instance();
+    if (!ocrManager->isAvailable()) {
+        return QVariantMap();
+    }
+
+    auto languageMap = ocrManager->availableLanguagesWithNames();
+    QVariantMap result;
+    for (auto it = languageMap.constBegin(); it != languageMap.constEnd(); ++it) {
+        result[it.key()] = it.value();
+    }
+    return result;
+}
+
+bool SpectacleCore::startOcrExtraction(const QString &languageCode)
+{
+    if (m_videoMode) {
+        return false;
+    }
+
+    const bool hasCaptureWindows = !CaptureWindow::instances().isEmpty();
+
+    if (hasCaptureWindows) {
+        auto selectionEditor = SelectionEditor::instance();
+        auto inlineMessages = InlineMessageModel::instance();
+
+        m_ocrExportInProgress = true;
+        if (!selectionEditor->acceptSelection(ExportManager::UserAction)) {
+            m_ocrExportInProgress = false;
+            inlineMessages->push(InlineMessageModel::Error, i18nc("@info", "Please select a region before extracting text"));
+            return false;
+        }
+
+        QMetaObject::invokeMethod(
+            this,
+            [this, languageCode]() {
+                performOcrExtraction(languageCode);
+            },
+            Qt::QueuedConnection);
+        return true;
+    }
+
+    return performOcrExtraction(languageCode);
+}
+
+bool SpectacleCore::performOcrExtraction(const QString &languageCode)
+{
+    auto ocrManager = OcrManager::instance();
+    auto inlineMessages = InlineMessageModel::instance();
+
+    if (!ocrManager->isAvailable()) {
+        inlineMessages->push(InlineMessageModel::Error, i18nc("@info", "OCR is not available."));
+        return false;
+    }
+
+    const QImage image = m_annotationDocument->renderToImage();
+    if (image.isNull()) {
+        inlineMessages->push(InlineMessageModel::Error, i18nc("@info", "No screenshot available."));
+        return false;
+    }
+
+    inlineMessages->push(InlineMessageModel::Copied, i18nc("@info", "Extracting text from image..."));
+
+    if (languageCode.isEmpty()) {
+        ocrManager->recognizeText(image);
+    } else {
+        ocrManager->recognizeTextWithLanguage(image, languageCode);
+    }
+
+    return true;
+}
+
 SpectacleCore::~SpectacleCore() noexcept
 {
     s_self = nullptr;
@@ -588,7 +788,7 @@ SpectacleCore::~SpectacleCore() noexcept
 SpectacleCore *SpectacleCore::instance()
 {
     if (!s_self) {
-        s_self = new SpectacleCore();
+        s_self = new SpectacleCore(qApp);
     }
 
     return s_self;
@@ -1034,8 +1234,6 @@ void SpectacleCore::showViewerIfGuiMode(bool minimized)
     }
 }
 
-static QList<KNotification *> notifications;
-
 void SpectacleCore::doNotify(ScreenCapture type, const ExportManager::Actions &actions, const QUrl &saveUrl)
 {
     if (m_cliOptions[CommandLineOptions::NoNotify]) {
@@ -1049,11 +1247,12 @@ void SpectacleCore::doNotify(ScreenCapture type, const ExportManager::Actions &a
 
     KNotification *notification = nullptr;
     QString title;
+    // Use nullptr as the notification parent to prevent them from disappearing from history.
     if (type == ScreenCapture::Screenshot) {
-        notification = new KNotification(u"newScreenshotSaved"_s, KNotification::CloseOnTimeout, this);
+        notification = new KNotification(u"newScreenshotSaved"_s, KNotification::CloseOnTimeout, nullptr);
         title = CaptureModeModel::captureModeLabel(toCaptureMode(m_lastGrabMode));
     } else {
-        notification = new KNotification(u"recordingSaved"_s, KNotification::CloseOnTimeout, this);
+        notification = new KNotification(u"recordingSaved"_s, KNotification::CloseOnTimeout, nullptr);
         title = RecordingModeModel::recordingModeLabel(m_lastRecordingMode);
     }
     notification->setTitle(title);
@@ -1068,14 +1267,14 @@ void SpectacleCore::doNotify(ScreenCapture type, const ExportManager::Actions &a
     if (type == ScreenCapture::Screenshot) {
         if (actions & Action::AnySave && !saveFileName.isEmpty()) {
             if (actions & Action::CopyPath) {
-                notification->setText(i18n("A screenshot was saved as '%1' to '%2' and the file path of the screenshot has been saved to your clipboard.",
+                notification->setText(i18nc("%1 is the filename, %2 the path to the save location", "A screenshot was saved as '%1' to '%2' and the file path of the screenshot has been saved to your clipboard.",
                                         saveFileName, saveDirPath));
             } else if (saveDirPath == QStandardPaths::writableLocation(QStandardPaths::PicturesLocation)) {
-                notification->setText(i18nc("Placeholder is filename",
+                notification->setText(i18nc("%1 is the filename",
                                             "A screenshot was saved as '%1' to your Pictures folder.",
                                             saveFileName));
             } else {
-                notification->setText(i18n("A screenshot was saved as '%1' to '%2'.",
+                notification->setText(i18nc("%1 is the filename, %2 the path to the save location", "A screenshot was saved as '%1' to '%2'.",
                                         saveFileName, saveDirPath));
             }
         } else if (actions & Action::CopyImage) {
@@ -1084,11 +1283,11 @@ void SpectacleCore::doNotify(ScreenCapture type, const ExportManager::Actions &a
     } else if (type == ScreenCapture::Recording && actions & Action::AnySave && !saveFileName.isEmpty()) {
         if (actions & Action::CopyPath) {
             notification->setText(
-                i18n("A recording was saved as '%1' to '%2' and the file path of the recording has been saved to your clipboard.", saveFileName, saveDirPath));
+                i18nc("%1 is the filename, %2 the path to the save location", "A recording was saved as '%1' to '%2' and the file path of the recording has been saved to your clipboard.", saveFileName, saveDirPath));
         } else if (saveDirPath == QStandardPaths::writableLocation(QStandardPaths::MoviesLocation)) {
-            notification->setText(i18nc("Placeholder is filename", "A recording was saved as '%1' to your Videos folder.", saveFileName));
+            notification->setText(i18nc("%1 is the filename", "A recording was saved as '%1' to your Videos folder.", saveFileName));
         } else {
-            notification->setText(i18n("A recording was saved as '%1' to '%2'.", saveFileName, saveDirPath));
+            notification->setText(i18nc("%1 is the filename, %2 the path to the save location", "A recording was saved as '%1' to '%2'.", saveFileName, saveDirPath));
         }
     }
 
